"estiml3m.E3.M3" <-
function(data,K=3,prop.init=c(0.3,0.3,0.4),mean.init=cbind(c(0.2,0,-1),c(0,-0.5,1),c(-0.2,0.5,1)),tau2.init=cbind(c(0.1,0.1),c(0.1,0.1),c(0.1,0.1)),sigma2.init=c(0.2,0.2,0.2),iter.em=150,impres=1,name.data=NULL,eps=0.00001)
  {
    
    #
    # Definition of model.E3.M3
    # (E3) : y_{itr}=\beta_t+e_{it}+e_{i}+\epsilon\{itr}
    # (M3) : (\beta_k,\tau^2_k,\sigma^2\k,k=1,\ldots,k=K)
    #
 
 

###
###  Definition of the variables for E3.M3 model

    require(mvtnorm) ||stop("The library mvtnorm is not installed")
    
    
    I<-dim(data)[1]
    Tp<-dim(data)[2]
    R<-dim(data)[3]
    N<-I*Tp*R
    prop.theta<-prop.init
    mean.theta<-mean.init
    tau2.theta<-tau2.init
    sigma2.theta<-sigma2.init
    if(is.null(name.data))
       name.data<-deparse(substitute(data))
    mat.data<-t(apply(data,1,function(x){t(x)} ))
        
    
    if(!is.array(data))
      {
        stop("Parameter data is not an array object.")
      }

    if(Tp!=dim(mean.init)[1] | K!=dim(mean.init)[2] |  dim(tau2.init)[2]!=K |  dim(tau2.init)[1]!=2 | length(sigma2.init)!=K )
      {
        stop("Error of dimension  for parameters initialisation.")
      }
    if(R==1)
      {
        stop("This model needs at least 2 repetitions.")
      }
    Xp<-matrix(0,nrow=Tp*R,ncol=Tp)
    Up<-NULL
    tmp<-rep(1,R)
    for(j in 1:Tp)
      {
        Xp[c(((j-1)*R+1):(j*R)),j]<-tmp
      }
 
    Up1<-rep(1,R*Tp)

    tmp<-rep(1,R)
    Up2<-diag(1,Tp)%x%tmp

    Up<-cbind(Up1,Up2)
    V<-list(NULL)
    V[[1]]<-Up1%*%t(Up1)
    V[[2]]<-Up2%*%t(Up2)
    
    
    Gamma<-gammal3m.E3.M3(R=R,Tp=Tp,K=K,tau2.theta=tau2.theta,sigma2=sigma2.theta)
    loglik<-0
    loglik.tmp<-0

    
#######################################
#######################################
####                               #### 
####     START EM ALGORITHM        ####
####                               ####
#######################################
#######################################
    
    for(l in 1:iter.em)
      {
               
#############################################################  
###            E step for EM algorithm                      #
###---------------------------------------------------------#
### We begin to calculate f_k(y_i)                          #   
### Results are stored in mat.fik matrix of (I,K) dimension #
#############################################################
 
        mat.fik<-matrix(0,nrow=I,ncol=K)
    
        for(k in 1:K)
          {
            muk<-Xp%*%mean.theta[,k]
            #for(i in 1:I)
            #  {
            #    y<-as.vector(t(data[i,,]))
            #    mat.fik[i,k]<-dmvnorm(y,muk,Gamma[[k]])            
            #  }# end for i in 1:I
            Gammak<-Gamma[[k]]
            mat.fik[,k]<-apply(mat.data,1,function(x,Gamma=Gammak,mu=muk) {dmvnorm(x,mu,Gamma)} )
          }#end for k in 1:K
       
    
        # We compute conditional probabilities t_{ik}.
        # Results are stored in mat.tik matrix of (I,K) dimension
        mat.propfik<-mat.fik%*%diag(prop.theta)
        den<-apply(mat.propfik,1,sum)
        mat.tik<-mat.propfik/den        


##########################################################  
###            M step for EM algorithm                   #
###------------------------------------------------------#
###   mean.new, tau2.new and sigma2.new are the          #   
###   new values of parameters.                          #
##########################################################
 
        
        
#### prop.theta ################
        prop.new<-apply(mat.tik,2,sum)/I
        if(sum(prop.new==0)>0)
          {
            cat("\n Problem during EM algorithm : class empty. Exit program L3M. \n")
            invisible(return(NULL))
            stop()
          }
        
        
#### sigma2.theta ###############
        sigma2.new<-sigma2.theta
        for(k in 1:K)
          {
            #m<-0
            mm<-0
            Gammakm1<-solve(Gamma[[k]])
            Gammakm2<-Gammakm1%*%Gammakm1       
            muk<-(Xp%*%mean.theta[,k])
            mat.data.moins.muk<-mat.data-rep(1,I)%*%t(muk)
            m2<-(R*Tp*sigma2.theta[k]) - ((sigma2.theta[k]^2)*sum(diag(Gammakm1)))
            #for(i in 1:I)
            #  {
            #    y<-as.vector(t(data[i,,]))               
            #    temp<-(y-muk)                
            #    m1<-(sigma2.theta[k]^2)*(t(temp)%*%Gammakm2%*%(temp))               
            #    m<-(m+mat.tik[i,k]*(m1+m2))
            #  }
            tmp<-apply(mat.data.moins.muk,1,function(x,Gamma.tmp=Gammakm2) {t(x) %*% Gamma.tmp %*% x})
            mm<-sum(mat.tik[,k]*((sigma2.theta[k])^2*tmp+m2))  
            sigma2.new[k]<-mm/(Tp*R*sum(mat.tik[,k]))            
          }
        
       
##### tau2.theta #############
        tau2.new<-tau2.theta
        dimeffet<-c(1,Tp)
        for(ef in 1:2)
          for(k in 1:K)
            {
              #m<-0
              mm<-0
              tau2k<-tau2.theta[ef,k]
              Gammakm1<-solve(Gamma[[k]])
              muk<-Xp%*%mean.theta[,k]
              mat.data.moins.muk<-mat.data-rep(1,I)%*%t(muk)
              m2<-(dimeffet[ef]*tau2k) - ( (tau2k^2)*sum(diag(Gammakm1%*%V[[ef]])))
              #for(i in 1:I)
              #  {
              #  y<-as.vector(t(data[i,,]))
              #  tmp<-(y-muk)               
              #  m1<-as.vector((Tp*tau2k^2)*(t(tmp)%*%Gammakm1%*%V[[ef]]%*%Gammakm1%*%(tmp)))
              #  m<-m+(mat.tik[i,k]*(m1+m2))
              #}
              Gammakm1.V<-Gammakm1%*%V[[ef]]%*%Gammakm1
              tmp<-apply(mat.data.moins.muk,1,function(x,Gamma.tmp=Gammakm1.V) {t(x) %*% Gamma.tmp %*% x})
              #mm<- sum(mat.tik[,k]* (Tp*tau2k^2*tmp+m2))
              mm<- sum(mat.tik[,k]* (tau2k^2*tmp+m2))
              tau2.new[ef,k]<-mm/(sum(mat.tik[,k])*dimeffet[ef])
              #print(paste("effet=",ef,"classe=",k))
              #print(mm-m)
              #print("")
            }
        
####### mean.theta ##############
        mean.new<-mean.theta
        Xpinv<-solve(t(Xp)%*%Xp)
        #Gammakm1<-solve(Gamma[[1]])
        for(k in 1:K)
          {
            mm<-0
            Gammakm1<-solve(Gamma[[k]])
            muk<-Xp%*%mean.theta[,k]
            mat.data.moins.muk<-mat.data-rep(1,I)%*%t(muk)
            #m<-0            
            #for(i in 1:I)
            #  {
            #    y<-as.vector(t(data[i,,]))
            #    tmp<-(y-Xp%*%mean.theta[,k])
            #    m<-m+mat.tik[i,k]*(sigma2.theta[k]*Xpinv%*%t(Xp)%*%Gammakm1%*%tmp+mean.theta[,k])
            #  }
            Gammakm1.Xp<-Xpinv%*%t(Xp)%*%Gammakm1
            tmp<-apply(mat.data.moins.muk,1,function(x,Gamma.tmp=Gammakm1.Xp) {Gamma.tmp%*%x})
            tmp<-sigma2.theta[k]*tmp+mean.theta[,k]            
            mm<-tmp%*%mat.tik[,k] 
            mean.new[,k]<-mm/sum(mat.tik[,k])
            #cat(paste(gsub(",","",c("Classe",k," : ",mm-m))))
            #cat("\n")
          }
        
        
#### We compute the log-likelihood for the iteration l in EM#####
        tmp<-apply(mat.propfik,1,sum)
        tmp<-log(tmp)        
        loglik<-sum(tmp,na.rm=T)
        if(abs(loglik-loglik.tmp)<eps) break
        loglik.tmp<-loglik

        
        
##### If impres is not equal to 0 and depending on the value of impres, ################
##### we display some results during the EM algorithm.  ################################
##### If impres equal to 1, few results are displayed ##################################
##### If impres equal to 2, all the results are displayed ##############################

        if(impres!=0)
          {
            if(impres==1)
              {
                
                cat(paste("Iteration= ",l,"\t---------\t","log-likelihood= ",loglik,"\r",sep=""))
              }
            if(impres==2)
              {
                cat("__________________________________________________________________________\n")
                cat(paste("Iteration= ",l,"\t ***** \t","log-likelihood= ",loglik,"\n"))
                cat("proportions = \n")
                print(round(prop.new,3))
                #cat("\n")
                cat("tau2.theta =\n ")
                print(round(tau2.new,3))
                #cat("\n")
                cat("sigma2.theta = \n")
                print(round(sigma2.new,3))
                #cat("\n")
                cat("mean.theta = \n")
                print(mean.new) 
              }
            
          }
        
#### We set the parameters  to the calculate values during EM 
        prop.theta<-prop.new
        tau2.theta<-tau2.new
        sigma2.theta<-sigma2.new
        mean.theta<-mean.new
        Gamma<-gammal3m.E3.M3(R=R,Tp=Tp,K=K,tau2.theta=tau2.theta,sigma2.theta=sigma2.theta)

        
      }#End of EM iterations

  #cat("\n")
    
  

    if(!is.null(dimnames(data)[[1]]))
      dimnames(mat.fik)<-list(dimnames(data)[[1]],1:K)
    
    
    invisible(list(model="E3.M3",name.data=name.data,mat.fik=mat.fik,prop.theta=prop.theta,mean.theta=mean.theta,tau2.theta=tau2.theta,sigma2.theta=sigma2.theta))
    

  }#fin du progrmme
