"init2" <-
function(data,model="E2.M2",K=3,init="kmeans",iter.em.init=20)
  {
    
    require(mva) ||stop("The library mva is not installed")
    require(nls) ||stop("The library nls is not installed")
    require(nlme) ||stop("The library nlme is not installed")
    I<-dim(data)[1]
    Tp<-dim(data)[2]
    R<-dim(data)[3]
    K<-K
    model<-model

    # We initialyze all the parameters.
    # It depends from the argument init
    # by default init="kmeans"
    # otherwise, init could be "hierarchical" or "mixte" (we combine "hierarchical" and "kmeans")

    # We make a vector with all the values
    # in the data array. The values are ordered following
    # y_{itr} principe. We make all the vectors for estim parameters
    # with lme program

    vec.data<-NULL
    for(i in 1:I)
      vec.data<-c(vec.data,as.vector(t(data[i,,])))
    #vec.Tp<-rep(1:Tp,rep(R,Tp))
    #vec.Tp<-rep(vec.Tp,I)
    #vec.I<-rep(1:I,rep(R*Tp,I))    
    #vec.IT<-rep(1:(I*Tp),rep(R,I*Tp))

     mean.data<-apply(data,c(1,2),mean)
    
    #############################
    ### Kmeans initialisation ###
    #############################
    if(init=="kmeans"|init==1)
      {
        if(K>1)
          {
            ##We compute nk kmeans
            nk<-20
            resinit<-list(NULL)
            inert.inter<-rep(0,nk)
            for(i in 1:nk)
              {                
                resinit[[i]]<-kmeans(mean.data,K,iter.max=30)
                if(any(summary(factor(resinit[[i]]$cluster))<=2))
                   {
                     inert.inter[i]<-0
                   }
                else
                  inert.inter[i]<-sum(resinit[[i]]$withinss*resinit[[i]]$size/sum(resinit[[i]]$size))
              }
            nomax<-which(inert.inter%in%max(inert.inter))[1]
            resinit<-resinit[[nomax]]                       
            kmeans.cluster<-resinit$cluster
            kmeans.cluster<-rep(kmeans.cluster,rep(R,I))
            means.centers<-resinit$centers
          }        
        #if(K==1)
        #  {
        #    kmeans.cluster<-rep(1,I)
        #    means.centers<-apply(data[,,1],2,mean)           
        #  }        
        gene.classe<-list(NULL)        
        vec.II<-rep(1:I,rep(R,I))
        for(k in 1:K)
          {
            gene.classe[[k]]<-unique(vec.II[kmeans.cluster==k])
          }         
      }

    ##################################
    ####Hierarchical initialisation###
    ##################################
    if(init=="hierarchical"|init==2)
      {
        D<-dist(mean.data)
        resinit<-hclust(D)
        rm(D)
        hc.cut<-cutree(resinit,k=K)
        hc.cut<-rep(hc.cut,rep(R,I))
        gene.classe<-list(NULL)
        vec.II<-rep(1:I,rep(R,I))
        means.centers<-matrix(0,nrow=K,ncol=Tp)
        for(k in 1:K)
          {
            gene.classe[[k]]<-unique(vec.II[hc.cut==k])
            means.centers[k,]<-apply(mean.data[gene.classe[[k]], ],2,mean)
          }
        gene.classe<-gene.classe
        
      }

    
    ############################
    ####Mixte initialisation####
    ############################
    if(init=="mixte"|init==3)
      {        
        D<-dist(mean.data)
        resinit<-hclust(D)
        rm(D)
        hc.cut<-cutree(resinit,k=K)
        hc.cut<-rep(hc.cut,rep(R,I))
        num.class<-hc.cut
        centre<-matrix(0,nrow=K,ncol=dim(mean.data)[2])
        for(i in 1:K)
        {
          tmp<-mean.data[hc.cut==i,]          
          centre[i,]<-apply(tmp,2,mean)
        }
        resinit<-kmeans(mean.data,centers=centre,iter.max=30)
        kmeans.cluster<-resinit$cluster
        means.centers<-resinit$centers
        gene.classe<-list(NULL)
        vec.II<-rep(1:I,rep(R,I))       
        for(k in 1:K)
          {
            gene.classe[[k]]<-unique(vec.II[kmeans.cluster==k])
          }
      }
    
    # In gene.classe[[k]], we store the numeros of genes that belong to the cluster k    

    ############################################################################
    ### Compute initialization parameters with the previous clsutering results##
    ### and with the lme function                                             ##
    ############################################################################
    if(init!="em" & init!="EM" & init!=4 & init!="em2" & init!="EM2" & init!=5)
      {
        resinitlme<-initlme(vec.data=vec.data,I=I,Tp=Tp,R=R,K=K,model=model,gene.classe=gene.classe)
        prop.init<-resinitlme$prop.init
        tau2.init<-resinitlme$tau2.init
        sigma2.init<-resinitlme$sigma2.init
        #mean.init<-resinitlme$mean.init
        mean.init<-t(means.centers)
      }
    ############################
    ####EM initialisation#######
    ############################

     if(init=="EM"|init=="em"|init==4)
      {
       # mean.data<-apply(data,c(1,2),mean)
        if(K>1)
          {
            ##We compute nk kmeans following with EM (l3m algorithm)
            loglik.em<--50000
            #nk.tot<-iter.em.init
            nk<-0
            resinit<-list(NULL)
            resinitlme<-list(NULL)
            gene.classe<-list(NULL)
            while(nk < iter.em.init)
              {
                #print(c("essai :",nk))
                resinit[[i]]<-kmeans(mean.data,K,iter.max=30)
                kmeans.cluster<-resinit[[i]]$cluster
                kmeans.cluster<-rep(kmeans.cluster,rep(R,I))
                means.centers<-resinit[[i]]$centers
                if(!any(summary(factor(resinit[[i]]$cluster))<=2))
                   {
                     vec.II<-rep(1:I,rep(R,I))
                     for(k in 1:K)
                       {
                         gene.classe[[k]]<-unique(vec.II[kmeans.cluster==k])
                       }
                     resinitlme<-initlme(vec.data=vec.data,I=I,Tp=Tp,R=R,K=K,model=model,gene.classe=gene.classe)
                     prop.initem<-resinitlme$prop.init
                     mean.initem<-t(means.centers)
                     #mean.initem<-resinitlme$mean.init
                     if(model !="E0.M1" & model !="E0.M3")
                       tau2.initem<-resinitlme$tau2.init
                     sigma2.initem<-resinitlme$sigma2.init

                     switch(model,
                            "E0.M1"={res<-estiml3m.E0.M1(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E0.M3"={res<-estiml3m.E0.M3(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            
                            "E1.M1"={res<-estiml3m.E1.M1(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E1.M2"={res<-estiml3m.E1.M2(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E1.M3"={res<-estiml3m.E1.M3(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
           
                            "E2.M1"={res<-estiml3m.E2.M1(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E2.M2"={res<-estiml3m.E2.M2(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E2.M3"={res<-estiml3m.E2.M3(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
           
                            "E3.M1"={res<-estiml3m.E3.M1(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E3.M2"={res<-estiml3m.E3.M2(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") },
                            "E3.M3"={res<-estiml3m.E3.M3(data=data,K=K,prop.init=prop.initem,mean.init=mean.initem,tau2.init=tau2.initem,sigma2.init=sigma2.initem,iter.em=10,impres=0,name.data="initdata") }
                            )

                     if(!is.null(res))
                       {                         
                         cat(paste("Iteration EM initialisation= ",nk, "\r"))
                         summaryres<-summaryl3m(res,printout=F)
                         #print(paste("Iteration : ",i,"  loglik.em=",summaryres["loglik"]))
                         if(summaryres["loglik"]>loglik.em)
                           {                        
                             loglik.em<-summaryres["loglik"]
                             #print(paste("nvelle vraisemblance",loglik.em))
                             prop.init<-res$prop.theta
                             mean.init<-res$mean.theta         
                             #mean.init<-t(means.centers)
                             if(model !="E0.M1" & model !="E0.M3")                      
                               tau2.init<-res$tau2.theta
                             else
                               tau2.init<-NULL
                             sigma2.init<-res$sigma2.theta                            
                           }
                        nk<-nk+1 
                       }
                   }#end if(!any...                               
              }#end loop for(i in 1:nk)  loops for nk em2                   
          }#end for if(K>1)       
      }#end for init em2

    
    invisible(list(vec.data=vec.data,prop.init=prop.init,tau2.init=tau2.init,sigma2.init=sigma2.init,mean.init=mean.init))
              
  }
