%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\name{l3m}

\alias{l3m}

\title{Parameter estimations for mixture of linear mixed models.}

\description{
 The function \code{l3m} allows parameter estimations for mixture of
 linear mixed models through the EM algorithm. Different mixed models
 (E1, E2, E3) and different mixture models (M1, M2, M3) are possible. A simpler linear
 model E0, with no random effect is also considered.
}

\usage{
l3m(data,K=2,model,impres=1,iter.em=150,eps=0.0001,init="em",
    param.init=NULL,iter.em.init=30)
}

\arguments{
 \item{data}{an array of integer values. The dimension of array is IxTpxR.}
 \item{K}{an integer strictly larger than 1 for the number of components for the mixture.}
 \item{model}{a character string which indicates the considered
   model. Possible models correspond to combinations of mixed models and
   mixture models, namely  "E0.M1", "E0.M3", "E1.M1", "E1.M2", "E1.M3", "E2.M1", "E2.M2",
 "E2.M3", "E3.M1", "E3.M2", "E3.M3".}
 \item{impres}{a value which specifies the informations printed in the R
   console during the algorithm. By default, impres is equal to 1 and
   only information on log-likelihood is printed. Others values are 0 and 2.}
 \item{iter.em}{an integer which gives the maximum number of iterations
   for EM algorithm. By default iter.em is equal to 150.}
 \item{eps}{a real number which stops the EM algorithm if the difference
 of the log-likelhihood between two iterations of the EM algorithm is
 less than eps. By default eps is equal to 0.0001. }
\item{init}{a character string for the EM initialzation. By default
  init="em". Others initilizations are "kmeans" or "hierarchical".}
 \item{param.init}{a list which gives a possible initialization for
  parameter estimations. If NULL, a random initialization is
  selected. By default param.init is NULL.}
 \item{iter.em.init}{an integer which gives the number for short EM runs
   when init="em". }
 }

\details{The array represents the R repeated measurements for I statistical
   units and for Tp variables. The dimension is IxTpxR. In gene expression profiles
   analysis, if each experiment is reproduced R times, this array is 
   the log-ratio expressions for I genes  measured at Tp times.
}

\value{
  \code{l3m}  returns a list containing the following components.
  
    \item{model}{the name of the mixture of linear mixed models.}
    \item{name.data}{the name of the array.}
    \item{mat.fik}{a matrix of dimension IxK which gives the density
      probabilities for each statistic unit and each component of the mixture. } 
    \item{prop.theta}{a vector for the proportion estimations of the mixture. }
    \item{mean.theta}{a matrix of dimension TpxK for the estimation of
      the fixed effects for the mixture components.}
    \item{tau2.theta}{a real (for mixed model E1), a vector (for mixed
      model E2) or a matrix (for mixed model E3) which give the
      variance estimations for the random effect. }
    \item{sigma2.theta}{a real (for mixture models M1 and M2) or a vector
      (for mixture model M3) for the variance estimation of the
      error measure.}
 }
   

\author{Martin Olivier, \email{martinol@ensam.inra.fr}}

\references{
\item G. McLachlan, D. Peel. Finite mixture models. Wiley, 2000.
\item S.R. Searle, G. Casella, C.E. Mc Culloch. Variance components. John Wiley & Sons, 1992.
}


\seealso{
  \code{\link{datal3m}}, \code{\link{partitionl3m}}, \code{\link{summaryl3m}}
}


\examples{
data(datal3m)

dim(datal3m)
## You will see: [1] 100   3   5  
## There are I=100 statistical units measured with three variables (Tp=3).
## Each measurement is realized R=5 times.

resl3m<-l3m(datal3m,K=3,model="E2.M3",init="em",iter.em.init=10,iter.em=50)
## The estimation for the model E2.M3 (mixed model E2 and mixture model M3)
## is performed through the EM algorithm.
## The strategy for EM initialisation consists of iter.em.init=10 short
## runs of EM, followed by a long run of EM (with iter.em=50 iterations)
## from the short run solution providing the highest log-likelihood.
}

\keyword{print}

